//Project: CNC v2
//Homepage: www.HomoFaciens.de
//Author Norbert Heinz
//Version: 1.1
//Creation date: 16.12.2015
//This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//For a copy of the GNU General Public License see http://www.gnu.org/licenses/
//
//compile with gcc commands-CNC.c -o commands-CNC -lm
//For details see:
//http://www.HomoFaciens.de/technics-machines-cnc-v2_en_navion.htm

#include <stdio.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <dirent.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>
#include <complex.h>



#define BAUDRATE B115200
//#define ARDUINOPORT "/dev/ttyUSB0"
#define ARDUINOPORT "/dev/ttyACM0"
#define FALSE 0
#define TRUE 1

//Number of bytes to sent via USB before waiting for 'r' from Arduino
//Must be
#define MAX_ARDUINO_BUFFER 1
#define MAX_FEED_RATE     500

#define PI 3.1415927

#define KEYHITWAIT 1

int KeyHit = 0;
int KeyCode[5];
int SingleKey=0;

FILE *GcodeFile;

int  MaxRows = 24;
int  MaxFileRows = 10;
int  FilesFound = 0;
int  MaxCols = 80;
int  MessageX = 1;
int  MessageY = 24;
double Scale = 1.0;
double OldScale = 1.0;
long MoveLength = 1;
long xMin = 1000000, xMax = -1000000;
long yMin = 1000000, yMax = -1000000;
long zMin = 1000000, zMax = -1000000;
char FileName[1000] = "";
long currentX = 0, currentY = 0, currentZ = 0, currentFeedRate = MAX_FEED_RATE, latestFeedRate = MAX_FEED_RATE;

double newX, newY, newZ, zStart = 50.0, feedRateNew;

int stopPlot = 0;


int SpindleOn = 0;
int CoolantOn = 0;
int PauseTime = 0;
char LastGCodeCharacter = 0;
int LastGcodeNumber = 0;
int MaxSpeed = 1;
int AbsolutePositioning = 1;

double StepsPermmX = 1200.0 / 10.00;
double StepsPermmY = 1200.0 / 10.00;
double StepsPermmZ = 1200.0 / 10.00;
long zHubUp = 0, zHubDown = -100;

int fd = 0;
struct termios TermOpt;
int waitForKeyHit = KEYHITWAIT;

int relayOn = 0;
int coolantOn = 0;
int OldStepPause = 0;
int StepPause = 100;
int StepPauseArduino = 1;

int plotterSteps = 0;

int arduinoBuffer = 0;


char PicturePath[1000];




//+++++++++++++++++++++++ Start gotoxy ++++++++++++++++++++++++++
//Thanks to 'Stack Overflow', found on http://www.daniweb.com/software-development/c/code/216326
int gotoxy(int x, int y) {
  char essq[100]; // String variable to hold the escape sequence
  char xstr[100]; // Strings to hold the x and y coordinates
  char ystr[100]; // Escape sequences must be built with characters

  //Convert the screen coordinates to strings.
  sprintf(xstr, "%d", x);
  sprintf(ystr, "%d", y);

  //Build the escape sequence (vertical move).
  essq[0] = '\0';
  strcat(essq, "\033[");
  strcat(essq, ystr);

  //Described in man terminfo as vpa=\E[%p1%dd. Vertical position absolute.
  strcat(essq, "d");

  //Horizontal move. Horizontal position absolute
  strcat(essq, "\033[");
  strcat(essq, xstr);
  // Described in man terminfo as hpa=\E[%p1%dG
  strcat(essq, "G");

  //Execute the escape sequence. This will move the cursor to x, y
  printf("%s", essq);
  return 0;
}
//------------------------ End gotoxy ----------------------------------

//+++++++++++++++++++++++ Start clrscr ++++++++++++++++++++++++++
void clrscr(int StartRow, int EndRow) {
  int i, i2;

  if (EndRow < StartRow){
    i = EndRow;
    EndRow = StartRow;
    StartRow = i;
  }
  gotoxy(1, StartRow);
  for (i = 0; i <= EndRow - StartRow; i++){
    for(i2 = 0; i2 < MaxCols; i2++){
      printf(" ");
    }
    printf("\n");
  }
}
//----------------------- End clrscr ----------------------------

//+++++++++++++++++++++++ Start kbhit ++++++++++++++++++++++++++++++++++
//Thanks to Undertech Blog, http://www.undertec.de/blog/2009/05/kbhit_und_getch_fur_linux.html
int kbhit(void) {

   struct termios term, oterm;
   int fd = 0;
   int c = 0;

   tcgetattr(fd, &oterm);
   memcpy(&term, &oterm, sizeof(term));
   term.c_lflag = term.c_lflag & (!ICANON);
   term.c_cc[VMIN] = 0;
   term.c_cc[VTIME] = waitForKeyHit;
   tcsetattr(fd, TCSANOW, &term);
   c = getchar();
   tcsetattr(fd, TCSANOW, &oterm);
   if (c != -1)
   ungetc(c, stdin);

   return ((c != -1) ? 1 : 0);

}
//------------------------ End kbhit -----------------------------------

//+++++++++++++++++++++++ Start getch ++++++++++++++++++++++++++++++++++
//Thanks to Undertech Blog, http://www.undertec.de/blog/2009/05/kbhit_und_getch_fur_linux.html
int getch(){
   static int ch = -1, fd = 0;
   struct termios new, old;

   fd = fileno(stdin);
   tcgetattr(fd, &old);
   new = old;
   new.c_lflag &= ~(ICANON|ECHO);
   tcsetattr(fd, TCSANOW, &new);
   ch = getchar();
   tcsetattr(fd, TCSANOW, &old);

//   printf("ch=%d ", ch);

   return ch;
}
//------------------------ End getch -----------------------------------

//++++++++++++++++++++++ Start MessageText +++++++++++++++++++++++++++++
void MessageText(char *message, int x, int y, int alignment){
  int i;
  char TextLine[300];

  clrscr(y, y);
  gotoxy (x, y);

  TextLine[0] = '\0';
  if(alignment == 1){
    for(i=0; i < (MaxCols - strlen(message)) / 2 ; i++){
      strcat(TextLine, " ");
    }
  }
  strcat(TextLine, message);

  printf("%s\n", TextLine);
}
//-------------------------- End MessageText ---------------------------

//++++++++++++++++++++++ Start PrintRow ++++++++++++++++++++++++++++++++
void PrintRow(char character, int y){
  int i;
  gotoxy (1, y);
  for(i=0; i<MaxCols;i++){
    printf("%c", character);
  }
}
//-------------------------- End PrintRow ------------------------------

//+++++++++++++++++++++++++ ErrorText +++++++++++++++++++++++++++++
void ErrorText(char *message){
  clrscr(MessageY + 2, MessageY + 2);
  gotoxy (1, MessageY + 2);
  printf("Last error: %s", message);
}
//----------------------------- ErrorText ---------------------------

//+++++++++++++++++++++++++ PrintMenue_01 ++++++++++++++++++++++++++++++
void PrintMenue_01(){
  char TextLine[300];

   clrscr(1, MessageY-2);
   MessageText("*** Main menu plotter ***", 1, 1, 1);
   sprintf(TextLine, "M            - toggle move length, current value = %ld step(s)", MoveLength);
   MessageText(TextLine, 10, 2, 0);
   MessageText("Cursor right - move plotter in positive X direction", 10, 3, 0);
   MessageText("Cursor left  - move plotter in negative X direction", 10, 4, 0);
   MessageText("Cursor up    - move plotter in positive Y direction", 10, 5, 0);
   MessageText("Cursor down  - move plotter in negative Y direction", 10, 6, 0);
   MessageText("Page up      - lift miller", 10, 7, 0);
   MessageText("Page down    - touch down miller", 10, 8, 0);
   sprintf(TextLine, "Z            - set Z hub of miller: %ld Steps HubU=%ld, HubD=%ld, Z=%ld", zHubUp-zHubDown, zHubUp, zHubDown, currentZ);
   MessageText(TextLine, 10, 9, 0);
   sprintf(TextLine, "- / +        - decrease or increase lowest point of miller for 0.5mm");
   MessageText(TextLine, 10, 10, 0);
   sprintf(TextLine, "A            - set advance rate if miller down (ms between steps): %d ms", StepPause);
   MessageText(TextLine, 10, 11, 0);
   if(relayOn == 0){
     sprintf(TextLine, "R            - Relay ON/OFF, currently: OFF");
   }
   else{
     sprintf(TextLine, "R            - Relay ON/OFF, currently: ON");
   }
   MessageText(TextLine, 10, 12, 0);

   if(coolantOn == 0){
     sprintf(TextLine, "C            - Coolant ON/OFF, currently: OFF");
   }
   else{
     sprintf(TextLine, "C            - Coolant ON/OFF, currently: ON");
   }
   MessageText(TextLine, 10, 13, 0);

   MessageText("G            - create gearwheel file (GearWheel.svg).", 10, 14, 0);
   sprintf(TextLine, "F            - choose file: \"%s\"", FileName);
   MessageText(TextLine, 10, 15, 0);
   sprintf(TextLine, "S            - scale: %0.4f. W = %0.2fcm, H = %0.2fcm", Scale, (xMax - xMin) * Scale / StepsPermmX / 10.0, (yMax - yMin) * Scale / StepsPermmY / 10.0);
   MessageText(TextLine, 10, 16, 0);
   MessageText("E            - move to End switches", 10, 17, 0);
   MessageText("P            - plot file", 10, 18, 0);

   MessageText("Esc          - leave program", 10, 19, 0);

}
//------------------------- PrintMenue_01 ------------------------------

//+++++++++++++++++++++++++ PrintMenue_02 ++++++++++++++++++++++++++++++
char *PrintMenue_02(int StartRow, int selected){
  char TextLine[300];
  char OpenDirName[1000];
  static char FileName[101];
  DIR *pDIR;
  struct dirent *pDirEnt;
  int i = 0;
  int Discard = 0;

  clrscr(1, MessageY-2);
  MessageText("*** Choose plotter file ***", 1, 1, 1);

  strcpy(OpenDirName, PicturePath);


  pDIR = opendir(OpenDirName);
  if ( pDIR == NULL ) {
    sprintf(TextLine, "Could not open directory '%s'!", OpenDirName);
    MessageText(TextLine, 1, 4, 1);
    getch();
    return( "" );
  }

  FilesFound = 0;
  pDirEnt = readdir( pDIR );
  while ( pDirEnt != NULL && i < MaxFileRows) {
    //printf("%s\n", pDirEnt->d_name);
    if(strlen(pDirEnt->d_name) > 3){
      if(memcmp(pDirEnt->d_name + strlen(pDirEnt->d_name)-6, ".gcode",6) == 0 || memcmp(pDirEnt->d_name + strlen(pDirEnt->d_name)-4, ".ngc",4) == 0 || memcmp(pDirEnt->d_name + strlen(pDirEnt->d_name)-3, ".nc",3) == 0){
        FilesFound++;
        if(Discard >= StartRow){
          if(i + StartRow == selected){
            sprintf(TextLine, ">%s<", pDirEnt->d_name);
            strcpy(FileName, pDirEnt->d_name);
          }
          else{
            sprintf(TextLine, " %s ", pDirEnt->d_name);
          }
          MessageText(TextLine, 1, 3 + i, 0);
          i++;
        }
        Discard++;
      }
    }
    pDirEnt = readdir( pDIR );
  }

  gotoxy(MessageX, MessageY + 1);
  printf("Choose file using up/down keys and confirm with 'Enter' or press 'Esc' to cancel.");


  return (FileName);
}
//------------------------- PrintMenue_02 ------------------------------


//+++++++++++++++++++++++++ PrintMenue_03 ++++++++++++++++++++++++++++++
void PrintMenue_03(char *FullFileName, long CurrentLine, long NumberOfLines, long CurrentX, long CurrentY, long CurrentZ, long StartTime){
  char TextLine[300];
  long CurrentTime, ProcessHours = 0, ProcessMinutes = 0, ProcessSeconds = 0;

   CurrentTime = time(0);

   CurrentTime -= StartTime;

   while (CurrentTime > 3600){
     ProcessHours++;
     CurrentTime -= 3600;
   }
   while (CurrentTime > 60){
     ProcessMinutes++;
     CurrentTime -= 60;
   }
   ProcessSeconds = CurrentTime;

   clrscr(1, MessageY - 2);
   MessageText("*** Plotting file ***", 1, 1, 1);

   sprintf(TextLine, "File name: %s", FullFileName);
   MessageText(TextLine, 10, 3, 0);
   sprintf(TextLine, "Total number of lines: %ld", NumberOfLines);
   MessageText(TextLine, 10, 4, 0);
   sprintf(TextLine, "Current Line(%ld): X=%ld, Y=%ld Z=%ld    ", CurrentLine, CurrentX, CurrentY, CurrentZ);
   MessageText(TextLine, 10, 5, 0);
   sprintf(TextLine, "Process time: %02ld:%02ld:%02ld", ProcessHours, ProcessMinutes, ProcessSeconds);
   MessageText(TextLine, 10, 6, 0);


}
//------------------------- PrintMenue_03 ------------------------------


//+++++++++++++++++++++++ Start readport ++++++++++++++++++++++++++
char  readport(void){
  int n;
    char buff;
  n = read(fd, &buff, 1);
  if(n > 0){
    return buff;
  }
  return 0;
}
//------------------------ End readport ----------------------------------

//+++++++++++++++++++++++ Start sendport ++++++++++++++++++++++++++
void sendport(unsigned char ValueToSend){
  int n;
  int BytesInBuffer;


//  ioctl(fd,TIOCOUTQ,&BytesInBuffer);
//  while(BytesInBuffer > MAX_ARDUINO_BUFFER){//Wait until bytes in write buffer are below maximum
//    ioctl(fd,TIOCOUTQ,&BytesInBuffer);
//  }

  n = write(fd, &ValueToSend, 1);

  if (n < 0){
    ErrorText("write() of value failed!\r");
  }
//  else{
//    arduinoBuffer++;
//    if(arduinoBuffer == MAX_ARDUINO_BUFFER){
      //while(readport() < 1);
      readport();
      //while(readport() != 0);
//      arduinoBuffer = 0;
//    }
//  }




}
//------------------------ End sendport ----------------------------------

//+++++++++++++++++++++++ Start openport ++++++++++++++++++++++++++
void openport(void){

    //fd = open(ARDUINOPORT, O_RDWR | O_NOCTTY | O_NDELAY);
    fd = open(ARDUINOPORT, O_RDWR);
    if (fd == -1)  {
      ErrorText("init_serialport: Unable to open port ");
    }

    if (tcgetattr(fd, &TermOpt) < 0) {
      ErrorText("init_serialport: Couldn't get term attributes");
    }
    speed_t brate = BAUDRATE; // let you override switch below if needed

    cfsetispeed(&TermOpt, brate);
    cfsetospeed(&TermOpt, brate);

    // 8N1
    TermOpt.c_cflag &= ~PARENB;
    TermOpt.c_cflag &= ~CSTOPB;
    TermOpt.c_cflag &= ~CSIZE;
    TermOpt.c_cflag |= CS8;
    // no flow control
    TermOpt.c_cflag &= ~CRTSCTS;

    TermOpt.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    TermOpt.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    TermOpt.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    TermOpt.c_cc[VMIN]  = 0;
    TermOpt.c_cc[VTIME] = 20;

    if( tcsetattr(fd, TCSANOW, &TermOpt) < 0) {
      ErrorText("init_serialport: Couldn't set term attributes");
    }

}
//------------------------ End openport ----------------------------------


//++++++++++++++++++++++++++++++ MakeStepX ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void MakeStepX(int direction){

  if(stopPlot == 0){
    currentX+=direction;
    plotterSteps++;

    if(direction < 0){
      sendport('a');
    }
    else{
      sendport('b');
    }
  }

  waitForKeyHit = 0;
  if(kbhit()){
    if(getch() == 27){
      stopPlot = 1;
    }
  }
  waitForKeyHit = KEYHITWAIT;

}

//++++++++++++++++++++++++++++++ MakeStepY ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void MakeStepY(int direction){

  currentY+=direction;
  plotterSteps++;

  if(stopPlot == 0){
    if(direction > 0){
      sendport('c');
    }
    else{
      sendport('d');
    }
  }

  waitForKeyHit = 0;
  if(kbhit()){
    if(getch() == 27){
      stopPlot = 1;
    }
  }
  waitForKeyHit = KEYHITWAIT;

}

//++++++++++++++++++++++++++++++ MakeStepZ ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void MakeStepZ(int direction){

  if(stopPlot == 0){
    currentZ+=direction;
    plotterSteps++;

    if(direction < 0){
      sendport('f');
    }
    else{
      sendport('e');
    }
  }

  waitForKeyHit = 0;
  if(kbhit()){
    if(getch() == 27){
      stopPlot = 1;
    }
  }
  waitForKeyHit = KEYHITWAIT;

}


//++++++++++++++++++++++++++++++++++++++ CalculatePlotter ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
int CalculatePlotter(long moveX, long moveY, long moveZ){
  char TextLine[1000] = "";
  double  tempX = 0, tempY = 0, tempZ = 0;
  double xDiff, yDiff, zDiff;
  long moveXdone = 0, moveYdone = 0, moveZdone = 0;
  long maxDiff = 0;
  int i;

  sprintf(TextLine, "Moving X: %ld, Moving Y: %ld, Moving Z: %ld", moveX, moveY, moveZ);
  MessageText(TextLine, MessageX, MessageY, 0);

  maxDiff = abs(moveX);

  if(abs(moveY) > maxDiff){
    maxDiff=abs(moveY);
  }

  if(abs(moveZ) > maxDiff){
    maxDiff=abs(moveZ);
  }


  xDiff = (double)moveX / (double)maxDiff;
  yDiff = (double)moveY / (double)maxDiff;
  zDiff = (double)moveZ / (double)maxDiff;


  for(i=0; i<maxDiff; i++){
    tempX += xDiff;
    tempY += yDiff;
    tempZ += zDiff;

    if(tempX >= 1.0){
      MakeStepX(1);
      tempX -= 1.0;
      moveXdone++;
    }
    if(tempX <= -1.0){
      MakeStepX(-1);
      tempX += 1.0;
      moveXdone++;
    }

    if(tempY >= 1.0){
      MakeStepY(1);
      tempY -= 1.0;
      moveYdone++;
    }
    if(tempY <= -1.0){
      MakeStepY(-1);
      tempY += 1.0;
      moveYdone++;
    }

    if(tempZ >= 1.0){
      MakeStepZ(1);
      tempZ -= 1.0;
      moveZdone++;
    }
    if(tempZ <= -1.0){
      MakeStepZ(-1);
      tempZ += 1.0;
      moveZdone++;
    }
  }

  while(moveXdone < abs(moveX)){
    if(moveX > 1){
      MakeStepX(1);
    }
    else{
      MakeStepX(-1);
    }
    moveXdone++;
  }

  while(moveYdone < abs(moveY)){
    if(moveY > 1){
      MakeStepY(1);
    }
    else{
      MakeStepY(-1);
    }
    moveYdone++;
  }

  while(moveZdone < abs(moveZ)){
    if(moveZ > 1){
      MakeStepZ(1);
    }
    else{
      MakeStepZ(-1);
    }
    moveZdone++;
  }



  waitForKeyHit = 0;
  if(kbhit()){
    if(getch() == 27){
      return 1;
    }
  }
  waitForKeyHit = KEYHITWAIT;
  return 0;
}
//-------------------------------------- CalculatePlotter --------------------------------------------------------

//+++++++++++++++++++++++++++++ get next line of Gcode commands ++++++++
long GetGcodeCoordinates(int mirrorX, int mirrorY, int mirrorZ){
  char TextLine[1000];
  long i;
  char a = 0;
  char GCodeCharacter;
  int EndOfLine = 0;
  double GCodeNumber = 0.0;
  char *pEnd;


  a=0;
  i=0;
  TextLine[0] = '\0';
  EndOfLine = 0;
  while(!(feof(GcodeFile)) && EndOfLine == 0){
    if(a == '1' || a == '2' || a == '3' || a == '4' || a == '5' || a == '6' || a == '7' || a == '8' || a == '9' || a == '0' || a == '.' || a == '-'){
      TextLine[i] = a;
      TextLine[i+1] = '\0';
      i++;
      if(i==900){
        ErrorText("Line in Gcode file too long!");
      }
    }
    else{
      if(i > 0){
        LastGcodeNumber = GCodeNumber;
        GCodeNumber = strtod(TextLine, &pEnd);
        if(GCodeCharacter == 'x' || GCodeCharacter == 'X'){
          newX = GCodeNumber * StepsPermmX;
          if (mirrorX == 1){
            newX = xMax - newX;
          }
        }
        if(GCodeCharacter == 'y' || GCodeCharacter == 'Y'){
          newY = GCodeNumber * StepsPermmY;
          if (mirrorY == 1){
            newY = yMax - newY;
          }
        }
        if(GCodeCharacter == 'z' || GCodeCharacter == 'Z'){
          newZ = GCodeNumber * StepsPermmZ;
        }
//        printf("GCodeNumber=%lf, GCodeCharacter='%c', TextLine = %s\n", GCodeNumber, GCodeCharacter, TextLine);// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        if(GCodeCharacter == 'G' || GCodeCharacter == 'g' && GCodeNumber == 90){
          AbsolutePositioning = 1;
        }

        if(GCodeCharacter == 'G' || GCodeCharacter == 'g' && GCodeNumber == 91){
          AbsolutePositioning = 0;
        }

        if(GCodeCharacter == 'G' || GCodeCharacter == 'g' && GCodeNumber == 0){
          feedRateNew = MAX_FEED_RATE;
        }

        if(GCodeCharacter == 'G' || GCodeCharacter == 'g' && GCodeNumber > 0 && GCodeNumber < 4){
          feedRateNew = latestFeedRate;
        }

        if(GCodeCharacter == 'F' || GCodeCharacter == 'f' && GCodeNumber > 0){
          feedRateNew = GCodeNumber;
          if(feedRateNew > MAX_FEED_RATE){
            feedRateNew = MAX_FEED_RATE;
          }
          latestFeedRate = feedRateNew;
        }

        if(GCodeCharacter == 'M' || GCodeCharacter == 'm' && GCodeNumber == 3){
          SpindleOn = 1;
        }
        if(GCodeCharacter == 'M' || GCodeCharacter == 'm' && GCodeNumber == 5){
          SpindleOn = 0;
        }

        if(GCodeCharacter == 'P' || GCodeCharacter == 'p'){
          if(LastGCodeCharacter == 'G' || LastGCodeCharacter == 'g' && LastGcodeNumber == 4){
            PauseTime = GCodeNumber;
          }
        }

      }
      i = 0;
      if(a != ' ' && a != ';' && a != 10 && a != 9 && a != 0){
        LastGCodeCharacter = GCodeCharacter;
        GCodeCharacter = a;
      }
    }
    if(a == ';'){//Discard until end of line
      while(!(feof(GcodeFile)) && a != 10){
        fread(&a, 1, 1, GcodeFile);
      }
    }
    if(a == 10){//End of line
      EndOfLine = 1;
    }
    else{
      fread(&a, 1, 1, GcodeFile);
    }
  }

  if(feof(GcodeFile)){
    return 0;
  }
  return 1;
}

//----------------------------- get next line of Gcode commands --------

//++++++++++++++++++++++++++++++++++++++ KeyControl ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void KeyControl(int action){
int i;

  i = 0;
  SingleKey = 1;
  KeyCode[0] = 0;
  KeyCode[1] = 0;
  KeyCode[2] = 0;
  KeyCode[3] = 0;
  KeyCode[4] = 0;
  KeyHit = 0;
  while (kbhit()){
    KeyHit = getch();
    KeyCode[i] = KeyHit;
    i++;
    if(i == 5){
      i = 0;
    }
    if(i > 1){
      SingleKey = 0;
    }
  }
  if(SingleKey == 0){
    KeyHit = 0;
  }

  if(action == 1 || action == 2){
    //Move X-axis
    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 68 && KeyCode[3] == 0 && KeyCode[4] == 0){
      CalculatePlotter(-MoveLength, 0, 0);
    }

    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 67 && KeyCode[3] == 0 && KeyCode[4] == 0){
      CalculatePlotter(MoveLength, 0, 0);
    }

    //Move Y-axis
    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 65 && KeyCode[3] == 0 && KeyCode[4] == 0){
      CalculatePlotter(0, -MoveLength, 0);
    }

    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 66 && KeyCode[3] == 0 && KeyCode[4] == 0){
      CalculatePlotter(0, MoveLength, 0);
    }

    //Pen UP/DOWN
    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 53 && KeyCode[3] == 126 && KeyCode[4] == 0){
      CalculatePlotter(0, 0, MoveLength);
    }

    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 54 && KeyCode[3] == 126 && KeyCode[4] == 0){
      CalculatePlotter(0, 0, -MoveLength);
    }

    if(KeyHit == 'x'){
      sendport('g');
    }
    if(KeyHit == 'X'){
      sendport('h');
    }



    if(KeyHit == 'm'){
      MoveLength *= 10;
      if(MoveLength == 100){
        MoveLength = 120;
      }
      if(MoveLength == 12000){
        MoveLength = 1;
      }

      PrintMenue_01();
    }

    if(KeyHit == '4'){
      StepPause = 4;
      while (StepPauseArduino < StepPause){
        sendport('S');//Raise step pause
        StepPauseArduino+=1;
      }

      while(StepPauseArduino > StepPause){
        sendport('s');//Reduce step pause
        StepPauseArduino-=1;
      }
    }


    if(KeyHit == '0'){
      StepPause = 0;
      while (StepPauseArduino < StepPause){
        sendport('S');//Raise step pause
        StepPauseArduino+=1;
      }

      while(StepPauseArduino > StepPause){
        sendport('s');//Reduce step pause
        StepPauseArduino-=1;
      }
    }

    if(KeyHit == '1'){
      StepPause = 1;
      while (StepPauseArduino < StepPause){
        sendport('S');//Raise step pause
        StepPauseArduino+=1;
      }

      while(StepPauseArduino > StepPause){
        sendport('s');//Reduce step pause
        StepPauseArduino-=1;
      }
    }

    if(KeyHit == '2'){
      StepPause = 2;
      while (StepPauseArduino < StepPause){
        sendport('S');//Raise step pause
        StepPauseArduino+=1;
      }

      while(StepPauseArduino > StepPause){
        sendport('s');//Reduce step pause
        StepPauseArduino-=1;
      }
    }

    if(KeyHit == 'c'){
      if(coolantOn == 1){
        sendport('p');//Turn coolant pump off
        coolantOn = 0;
      }
      else{
        sendport('P');//Turn coolant pump on
        coolantOn = 1;
      }
      PrintMenue_01();
    }

    if(KeyHit == 'r'){
      if(relayOn == 1){
        sendport('r');//Turn relay off
        relayOn = 0;
      }
      else{
        sendport('R');//Turn relay on
        relayOn = 1;
      }
      PrintMenue_01();
    }

  }


}
//-------------------------------------- KeyControl --------------------------------------------------------


//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  int MenueLevel = 0;
  char FullFileName[1000] = "";
  char FileNameOld[1000] = "";
  struct winsize terminal;
  int i;
  int FileSelected = 0;
  int FileStartRow = 0;
  char *pEnd;
  char TextLine[30000];
  long coordinateCount = 0;
  long maxCoordinateCount = 0;
  char a;
  int ReadState = 0;
  long PlotStartTime = 0;


  openport();

  printf("\n\nWaiting for 'X' from Arduino (Arduino pluged in?)...\n");

  //Wait for 'X' from Arduino
  while(readport() != 'X');


  StepPause = 1000.0 / (StepsPermmX * MAX_FEED_RATE  / 60.0);

  while (StepPauseArduino < StepPause){
    sendport('S');//Raise step pause
    StepPauseArduino+=1;
  }

  while(StepPauseArduino > StepPause){
    sendport('s');//Reduce step pause
    StepPauseArduino-=1;
  }


  printf("\n\nStepPause=%d, StepPauseArduino=%d\n", StepPause, StepPauseArduino);
  getch();


  strcpy(FileName, "noFiLE");

  getcwd(PicturePath, 1000);
  strcat(PicturePath, "/files");
  printf("PicturePath=>%s<", PicturePath);

  if(ioctl(STDOUT_FILENO, TIOCGWINSZ, &terminal)<0){
    printf("Can't get size of terminal window");
  }
  else{
    MaxRows = terminal.ws_row;
    MaxCols = terminal.ws_col;
    MessageY = MaxRows-3;
  }

  clrscr(1, MaxRows);
  PrintRow('-', MessageY - 1);
  PrintMenue_01();

  MaxFileRows = MaxRows - 10;

  while (1){
    MessageText("Waiting for key press.", MessageX, MessageY, 0);

    if(MenueLevel == 0){
      KeyControl(1);

      if(KeyHit == 'f'){
        strcpy(FileNameOld, FileName);
        strcpy(FileName, PrintMenue_02(FileStartRow, FileSelected));
        MenueLevel = 1;
      }

      if(KeyHit == 's'){
        OldScale = Scale;
        MessageText("Type new scale value: ", 1, MessageY, 0);
        gotoxy(23, MessageY);
        scanf("%lf", &Scale);
        if(Scale == 0){
          Scale = OldScale;
        }
        else{
          PrintMenue_01();
        }
      }

      if(KeyHit == 'e'){//Move to end switches
      int xSwitch = 0;
      int ySwitch = 0;
      int zSwitch = 0;
      int n;
      char readSwitch = 0;
      unsigned char buff = 'Q';

        while (zSwitch == 0){
          buff = 'Q';
          n = write(fd, &buff, 1);

          if (n < 0){
            ErrorText("write() of value failed!\r");
          }
          else{
            readSwitch = 0;
            while(readSwitch != 'Z' && readSwitch != 'z'){
              readSwitch = readport();
              if(readSwitch == 'Z'){
                zSwitch = 1;
              }
            }
          }
          if(zSwitch == 0){
            CalculatePlotter(0, 0, 1);
          }
        }

        while (ySwitch == 0){
          buff = 'Q';
          n = write(fd, &buff, 1);

          if (n < 0){
            ErrorText("write() of value failed!\r");
          }
          else{
            readSwitch = 0;
            while(readSwitch != 'Z' && readSwitch != 'z'){
              readSwitch = readport();
              if(readSwitch == 'Y'){
                ySwitch = 1;
              }
            }
          }
          if(ySwitch == 0){
            CalculatePlotter(0, -1, 0);
          }
        }

        while (xSwitch == 0){
          buff = 'Q';
          n = write(fd, &buff, 1);

          if (n < 0){
            ErrorText("write() of value failed!\r");
          }
          else{
            readSwitch = 0;
            while(readSwitch != 'Z' && readSwitch != 'z'){
              readSwitch = readport();
              if(readSwitch == 'X'){
                xSwitch = 1;
              }
            }
          }
          if(xSwitch == 0){
            CalculatePlotter(-1, 0, 0);
          }
        }


      }



      if(KeyHit == '+'){
        zHubDown+=StepsPermmZ / 5.0;
        PrintMenue_01();
      }

      if(KeyHit == '-'){
        zHubDown-=StepsPermmZ / 5.0;
        PrintMenue_01();
      }



      if(KeyHit == 'd'){//Create sensor disc
        FILE *FileOut;
        long double toothDiameter = 0.0;
        long double gearTeeth = 0.0;
        long double gearHole = 0.0;
        long double gearMiller = 0.0;
        long double r = 0.0, r1 = 0.0;
        long double x, y, Zrun, tempVar;
        long double alpha;
        long double beta;
        long double slitAngle, millerAngle;
        long double materialThickness = 0.0;
        long double deeperPerRun = 0.0;
        long double slitLength = 4.0;
        char Zeile[100000];
        char fileName[10000];

        MessageText("Type tooth width in mm: ", 1, MessageY, 0);
        gotoxy(28, MessageY);
        scanf("%Lf", &toothDiameter);
        if(toothDiameter > 0.0){
          MessageText("Type number of teeth: ", 1, MessageY, 0);
          gotoxy(23, MessageY);
          scanf("%Lf", &gearTeeth);
          gearTeeth = (int)gearTeeth;
          if(gearTeeth > 0.0){
            MessageText("Type diameter of center hole in mm: ", 1, MessageY, 0);
            gotoxy(37, MessageY);
            scanf("%Lf", &gearHole);
            gearHole /= 2.0;
            if(gearHole > 0.0){
              MessageText("Type diameter of miller in mm: ", 1, MessageY, 0);
              gotoxy(32, MessageY);
              scanf("%Lf", &gearMiller);
              gearMiller /= 2.0;
              if(gearMiller > 0.0){
                MessageText("Type material thickness in mm: ", 1, MessageY, 0);
                gotoxy(32, MessageY);
                scanf("%Lf", &materialThickness);
                if(materialThickness > 0.0){
                  MessageText("Type Z diff per run in mm: ", 1, MessageY, 0);
                  gotoxy(32, MessageY);
                  scanf("%Lf", &deeperPerRun);
                  if(deeperPerRun > 0.0){
                    strcpy(fileName, PicturePath);
                    strcat(fileName, "/SensorWheel.nc");

                    double r_a, r_b;
                    r_a = sinl(PI / (gearTeeth * 4.0)) * toothDiameter / 2.0;
                    r_b = (cosl(PI / (gearTeeth * 4.0)) * toothDiameter / 2.0) / (tanl(PI / (gearTeeth * 4.0)));
                    r = r_a + r_b;

                    if((FileOut=fopen(fileName,"wb"))==NULL){
                      printf("Can't open file '%s'!\n", fileName);
                      return(1);
                    }
                    else{






                      //Create path of central hole
                      r = gearTeeth * toothDiameter / PI + 1.0 + gearMiller / 2.0;
                      r1 = r - 1.0 - gearMiller / 2.0;

                      sprintf(Zeile, " ; Center hole\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      x = sinl(0.0) * (gearHole - gearMiller);
                      y = cosl(0.0) * (gearHole - gearMiller);
                      sprintf(Zeile, "G0 Z6.000\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      sprintf(Zeile, "G0 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      Zrun = 0.0;
                      int firstHolePath = 1;
                      while(cabs(Zrun) < materialThickness){
                        Zrun -= deeperPerRun;
                        if(cabs(Zrun) > materialThickness){
                          Zrun = -materialThickness;
                        }
                        if(firstHolePath == 0){
                          sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }
                        for(alpha = 0.0; alpha < 2.0 * PI; alpha+=0.05){
                          x = sinl(alpha) * (gearHole - gearMiller);
                          y = cosl(alpha) * (gearHole - gearMiller);
                          if(firstHolePath == 1){
                            firstHolePath = 0;
                            sprintf(Zeile, "G0 Z0.000\n");
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }
                          sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }
                        x = sinl(0.0) * (gearHole - gearMiller);
                        y = cosl(0.0) * (gearHole - gearMiller);
                        sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      }//while(cabs(Zrun) < materialThickness){

                      sprintf(Zeile, "G0 Z6.000\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);


                      //Create slits
                      slitAngle = PI / gearTeeth;
                      millerAngle = sinl(gearMiller / r1);

                      sprintf(Zeile, " ; Slits, slitAngle = %LF\n", slitAngle);
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      beta = 0.0;

                      while(beta <= 2.0 * (PI - slitAngle)){

                        x = sinl(0.0) * (r1);
                        y = cosl(0.0) * (r1);
                        tempVar = x * cos(beta) - y * sin(beta);
                        y = y * cos(beta) + x * sin(beta);
                        x = tempVar;

                        sprintf(Zeile, "G0 Z6.000\n");
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        sprintf(Zeile, "G0 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);

                        Zrun = 0.0;
                        firstHolePath = 1;
                        long double tempR;

                        while(cabs(Zrun) < materialThickness){
                          Zrun -= deeperPerRun;
                          if(cabs(Zrun) > materialThickness){
                            Zrun = -materialThickness;
                          }
                          if(firstHolePath == 0){
                            sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }
                          for(alpha = 0.0; alpha <= slitAngle - millerAngle; alpha+=0.05){
                            if(alpha > slitAngle - millerAngle){
                              alpha = slitAngle - millerAngle;
                            }
                            x = sinl(alpha) * r1;
                            y = cosl(alpha) * r1;
                            tempVar = x * cos(beta) - y * sin(beta);
                            y = y * cos(beta) + x * sin(beta);
                            x = tempVar;
                            if(firstHolePath == 1){
                              firstHolePath = 0;
                              sprintf(Zeile, "G0 Z0.000\n");
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                              sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            }
                            sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }

                          tempR = r1 - slitLength;

                          for(alpha = slitAngle - millerAngle; alpha >= 0.0; alpha-=0.05){
                            if(alpha < 0.0){
                              alpha = 0.0;
                            }
                            x = sinl(alpha) * tempR;
                            y = cosl(alpha) * tempR;
                            tempVar = x * cos(beta) - y * sin(beta);
                            y = y * cos(beta) + x * sin(beta);
                            x = tempVar;
                            if(firstHolePath == 1){
                              firstHolePath = 0;
                              sprintf(Zeile, "G0 Z0.000\n");
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                              sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            }
                            sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }


                          x = sinl(0.0) * r1;
                          y = cosl(0.0) * r1;
                          tempVar = x * cos(beta) - y * sin(beta);
                          y = y * cos(beta) + x * sin(beta);
                          x = tempVar;
                          sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }//while(cabs(Zrun) < materialThickness){

                        x = sinl(0.0) * r1;
                        y = cosl(0.0) * r1;
                        tempVar = x * cos(beta) - y * sin(beta);
                        y = y * cos(beta) + x * sin(beta);
                        x = tempVar;
                        sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);

                        sprintf(Zeile, "G0 Z6.000\n");
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);


                        beta += slitAngle * 2.0;
                      }//while(beta < 2.0 * PI){


                      //Cutting line sensor wheel

                      sprintf(Zeile, " ; Center hole\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      x = sinl(0.0) * (r);
                      y = cosl(0.0) * (r);
                      sprintf(Zeile, "G0 Z6.000\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      sprintf(Zeile, "G0 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      Zrun = 0.0;
                      firstHolePath = 1;
                      while(cabs(Zrun) < materialThickness){
                        Zrun -= deeperPerRun;
                        if(cabs(Zrun) > materialThickness){
                          Zrun = -materialThickness;
                        }
                        if(firstHolePath == 0){
                          sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }
                        for(alpha = 0.0; alpha < 2.0 * PI; alpha+=0.05){
                          x = sinl(alpha) * (r);
                          y = cosl(alpha) * (r);
                          if(firstHolePath == 1){
                            firstHolePath = 0;
                            sprintf(Zeile, "G0 Z0.000\n");
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }
                          sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }
                        x = sinl(0.0) * (r);
                        y = cosl(0.0) * (r);
                        sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + r), (y + r));
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      }//while(cabs(Zrun) < materialThickness){

                      sprintf(Zeile, "G0 Z6.000\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);



                    }
                    fclose(FileOut);
                    while(kbhit()){
                      getch();
                    }
                    MessageText("File 'GearWheel.svg successfully created. Press any key to continue...", 1, MessageY, 0);
                    getch();
                  }
                }
              }
            }
          }
        }




      }

      if(KeyHit == 'g'){//Create gearwheel
        FILE *FileOut;
        long double gearModule = 0.0;
        long double toothDiameter = 0.0;
        long double gearTeeth = 0.0;
        long double gearHole = 0.0;
        long double gearMiller = 0.0;
        long double r = 0.0;
        long double x, y;
        long double alpha = 0;
        long double beta = 0.0;
        char Zeile[100000];
        char fileName[10000];
        long double faktor = 100.0;
        long double lastX, lastY;
        long double firstX, firstY;
        long double closePathX, closePathY;
        long double i = 0.0;
        long double averageX = 0.0, averageY = 0.0, coordNumber = 0.0;
        long double materialThickness = 0.0;
        long double deeperPerRun = 0.0;
        long double minX = 9999999.0f, minY = 9999999.0f;
        long double maxX = -9999999.0f, maxY = -9999999.0f;
        long double Zrun = 0.0;
        int spokes = 0;

        MessageText("Type tooth diameter in mm: ", 1, MessageY, 0);
        gotoxy(28, MessageY);
        scanf("%Lf", &toothDiameter);
        toothDiameter /= 2.0;
        if(toothDiameter > 0.0){
          MessageText("Type number of teeth: ", 1, MessageY, 0);
          gotoxy(23, MessageY);
          scanf("%Lf", &gearTeeth);
          gearTeeth = (int)gearTeeth;
          if(gearTeeth > 0.0){
            MessageText("Type diameter of center hole in mm: ", 1, MessageY, 0);
            gotoxy(37, MessageY);
            scanf("%Lf", &gearHole);
            gearHole /= 2.0;
            if(gearHole > 0.0){
              MessageText("Type diameter of miller in mm: ", 1, MessageY, 0);
              gotoxy(32, MessageY);
              scanf("%Lf", &gearMiller);
              gearMiller /= 2.0;
              if(gearMiller > 0.0){
                MessageText("Type material thickness in mm: ", 1, MessageY, 0);
                gotoxy(32, MessageY);
                scanf("%Lf", &materialThickness);
                if(materialThickness > 0.0){
                  MessageText("Type Z diff per run in mm: ", 1, MessageY, 0);
                  gotoxy(32, MessageY);
                  scanf("%Lf", &deeperPerRun);
                  if(deeperPerRun > 0.0){

                    MessageText("Cut spokes (Y/N)?: ", 1, MessageY, 0);
                    gotoxy(32, MessageY);
                    KeyHit = 0;
                    while(KeyHit != 'y' && KeyHit != 'n'){
                      KeyHit = getch();
                      if(KeyHit == 'y'){
                        spokes = 1;
                      }
                    }

                    strcpy(fileName, PicturePath);
                    strcat(fileName, "/GearWheel.nc");

                    double r_a, r_b;
                    r_a = sinl(PI / (gearTeeth * 4.0)) * toothDiameter / 2.0;
                    r_b = (cosl(PI / (gearTeeth * 4.0)) * toothDiameter / 2.0) / (tanl(PI / (gearTeeth * 4.0)));
                    r = r_a + r_b;

                    if((FileOut=fopen(fileName,"wb"))==NULL){
                      printf("Can't open file '%s'!\n", fileName);
                      return(1);
                    }
                    else{
                      strcpy(Zeile, "G21 ; Unit: mm\nG90 ; Absolute coordinates\nG1 Z6.000 F228.6\nG0 X0.000 Y0.000");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      //++++++++++ calculate average x and y for centering drill hole
                      firstX = 0.0;
                      firstY = 0.0;
                      lastX = 0.0;
                      lastY = 0.0;
                      closePathX = -99999.9;
                      closePathY = -99999.9;
                      for(i = 0.0; i < gearTeeth; i+=1.0){
                        beta =  0.0 - PI / gearTeeth / 8.0 + i * 2.0 * PI / gearTeeth;
                        r = (toothDiameter + gearMiller);
                        for(alpha = 0.0 - PI / gearTeeth / 2.0 + beta; alpha < PI + PI / gearTeeth / 2.0 + beta; alpha+=0.1){
                          if(firstX == -99999.0 && firstY == -99999.0){
                            firstX = r * cosl(alpha);
                            firstY = r * sinl(alpha);
                          }
                          x = r * cosl(alpha) + lastX - firstX;
                          y = r * sinl(alpha) + lastY - firstY;
                          if(closePathX == -99999.9 && closePathY == -99999.9){
                            closePathX = x;
                            closePathY = y;
                          }
                          if(x < minX){
                            minX = x;
                          }
                          if(y < minY){
                            minY = y;
                          }
                          if(x > maxX){
                            maxX = x;
                          }
                          if(y > maxY){
                            maxY = y;
                          }
                        }
                        x = r * cosl(PI + PI / gearTeeth + beta) + lastX - firstX;
                        y = r * sinl(PI + PI / gearTeeth + beta) + lastY - firstY;
                        if(x < minX){
                          minX = x;
                        }
                        if(y < minY){
                          minY = y;
                        }
                        if(x > maxX){
                          maxX = x;
                        }
                        if(y > maxY){
                          maxY = y;
                        }
                        lastX = x;
                        lastY = y;
                        firstX = 0.0;
                        firstY = 0.0;
                        r = (toothDiameter - gearMiller);
                        for(alpha = 2.0 * PI - PI / gearTeeth / 2.0 + PI / gearTeeth + beta; alpha > PI + PI / gearTeeth / 2.0 + PI / gearTeeth + beta; alpha-=0.1){
                          if(firstX == 0.0 && firstY == 0.0){
                            firstX = r * cosl(alpha);
                            firstY = r * sinl(alpha);
                          }
                          x = r * cosl(alpha) + lastX - firstX;
                          y = r * sinl(alpha) + lastY - firstY;
                          if(x < minX){
                            minX = x;
                          }
                          if(y < minY){
                            minY = y;
                          }
                          if(x > maxX){
                            maxX = x;
                          }
                          if(y > maxY){
                            maxY = y;
                          }
                        }
                        x = r * cosl(PI + PI / gearTeeth / 2.0 + PI / gearTeeth + beta) + lastX - firstX;
                        y = r * sinl(PI + PI / gearTeeth / 2.0 + PI / gearTeeth + beta) + lastY - firstY;
                        lastX = x;
                        lastY = y;
                        averageX+=x;
                        averageY+=y;
                        coordNumber+=1.0;
                        firstX = -99999.0;
                        firstY = -99999.0;
                      }//for(i = 0.0; i < gearTeeth; i+=1.0){

                      x=closePathX;
                      y=closePathY;

                      averageX = averageX / coordNumber;
                      averageY = averageY / coordNumber;
                      //---------- calculate average x and y for centering drill hole


                      //Create path of central hole
                      int firstHolePath = 1;
                      sprintf(Zeile, " ");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      x = sinl(0.0) * (gearHole - gearMiller);
                      y = cosl(0.0) * (gearHole - gearMiller);
                      sprintf(Zeile, "G0 Z6.000\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      sprintf(Zeile, "G0 X%0.3Lf Y%0.3Lf\n", (x + averageX - minX), (y + averageY - minY));
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      Zrun = 0.0;
                      while(cabs(Zrun) < materialThickness){
                        Zrun -= deeperPerRun;
                        if(cabs(Zrun) > materialThickness){
                          Zrun = -materialThickness;
                        }
                        if(firstHolePath == 0){
                          sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }
                        for(alpha = 0.0; alpha < 2.0 * PI; alpha+=0.05){
                          x = sinl(alpha) * (gearHole - gearMiller);
                          y = cosl(alpha) * (gearHole - gearMiller);
                          if(firstHolePath == 1){
                            firstHolePath = 0;
                            sprintf(Zeile, "G0 Z0.000\n");
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }
                          sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + averageX - minX), (y + averageY - minY));
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }
                        x = sinl(0.0) * (gearHole - gearMiller);
                        y = cosl(0.0) * (gearHole - gearMiller);
                        sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x + averageX - minX), (y + averageY - minY));
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);
                      }

                      sprintf(Zeile, "G0 Z6.000\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      if(spokes==1){

                        r_a = sinl(PI / (gearTeeth * 4.0)) * toothDiameter / 2.0;
                        r_b = (cosl(PI / (gearTeeth * 4.0)) * toothDiameter / 2.0) / (tanl(PI / (gearTeeth * 4.0)));
                        //r = r_a + r_b;
                        r = gearTeeth * toothDiameter * 2.0 / PI;

                        sprintf(Zeile, ";spokes\n");
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);

                        //Create paths of spokes hole
                        int firstHolePath = 1;
                        long double spokesHoleR = 0.0;
                        long double tempX, tempY, tempVar;
                        double alphaSpokes = 0.0;
                        int i;
                        int spokesNumber = 4;

                        spokesHoleR = (r - gearHole - toothDiameter) * 0.4;
                        sprintf(Zeile, ";r = %LF, spokesHoleR = %Lf,  gearHole = %LF, toothDiameter = %LF\n", r, spokesHoleR, gearHole, toothDiameter);
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);

                        spokesNumber = (gearHole + (r - gearHole - toothDiameter) / 2.0) * PI * 2.0 / (spokesHoleR * 2.4);

                        for(i = 0; i < spokesNumber; i++){
                          alphaSpokes += PI * 2.0 / spokesNumber;
                          firstHolePath = 1;

                          x = sinl(0.0) * (spokesHoleR - gearMiller);
                          y = cosl(0.0) * (spokesHoleR - gearMiller);
                          tempX = x;
                          tempY = y + gearHole + (r - gearHole - toothDiameter) / 2.0;

                          tempVar = tempX * cos(alphaSpokes) - tempY * sin(alphaSpokes);
                          tempY = tempY * cos(alphaSpokes) + tempX * sin(alphaSpokes);
                          tempX = tempVar;

                          sprintf(Zeile, "G0 Z6.000\n");
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          sprintf(Zeile, "G0 X%0.3Lf Y%0.3Lf\n", tempX + averageX - minX, tempY + averageY - minY);
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          Zrun = 0.0;
                          while(cabs(Zrun) < materialThickness){
                            Zrun -= deeperPerRun;
                            if(cabs(Zrun) > materialThickness){
                              Zrun = -materialThickness;
                            }
                            if(firstHolePath == 0){
                              sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            }
                            for(alpha = 0.0; alpha < 2.0 * PI; alpha+=0.05){
                              x = sinl(alpha) * (spokesHoleR - gearMiller);
                              y = cosl(alpha) * (spokesHoleR - gearMiller);
                              if(firstHolePath == 1){
                                firstHolePath = 0;
                                sprintf(Zeile, "G0 Z0.000\n");
                                fwrite(Zeile, 1, strlen(Zeile), FileOut);
                                sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                                fwrite(Zeile, 1, strlen(Zeile), FileOut);
                              }
                              tempX = x;
                              tempY = y + gearHole + (r - gearHole - toothDiameter) / 2.0;
                              tempVar = tempX * cos(alphaSpokes) - tempY * sin(alphaSpokes);
                              tempY = tempY * cos(alphaSpokes) + tempX * sin(alphaSpokes);
                              tempX = tempVar;
                              sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", tempX + averageX - minX, tempY + averageY - minY);
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            }
                            x = sinl(0.0) * (spokesHoleR - gearMiller);
                            y = cosl(0.0) * (spokesHoleR - gearMiller);
                            tempX = x;
                            tempY = y + gearHole + (r - gearHole - toothDiameter) / 2.0;
                            tempVar = tempX * cos(alphaSpokes) - tempY * sin(alphaSpokes);
                            tempY = tempY * cos(alphaSpokes) + tempX * sin(alphaSpokes);
                            tempX = tempVar;
                            sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", tempX + averageX - minX, tempY + averageY - minY);
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }

                          sprintf(Zeile, "G0 Z6.000\n");
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }//for(alphaSpokes = 0.0; alphaSpokes < PI; alphaSpokes += PI / 4.0){
                      }//if(spokes==1){


                      //Create path of gears

                      Zrun = 0.0;
                      int firstGearPath = 1;
                      while(cabs(Zrun) < materialThickness){
                        Zrun -= deeperPerRun;
                        if(cabs(Zrun) > materialThickness){
                          Zrun = -materialThickness;
                        }
                        if(firstGearPath == 0){
                          sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                        }
                        firstX = 0.0;
                        firstY = 0.0;
                        lastX = 0.0;
                        lastY = 0.0;
                        closePathX = -99999.9;
                        closePathY = -99999.9;
                        for(i = 0.0; i < gearTeeth; i+=1.0){
                          beta =  0.0 - PI / gearTeeth / 8.0 + i * 2.0 * PI / gearTeeth;
                          r = (toothDiameter + gearMiller);
                          for(alpha = 0.0 - PI / gearTeeth / 2.0 + beta; alpha < PI + PI / gearTeeth / 2.0 + beta; alpha+=0.1){
                            if(firstX == -99999.0 && firstY == -99999.0){
                              firstX = r * cosl(alpha);
                              firstY = r * sinl(alpha);
                            }
                            x = r * cosl(alpha) + lastX - firstX;
                            y = r * sinl(alpha) + lastY - firstY;
                            if(firstGearPath == 1){
                              sprintf(Zeile, "G0 X%0.3Lf Y%0.3Lf\n", (x - minX), (y - minY));
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                              sprintf(Zeile, "G0 Z0.000\n");
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                              sprintf(Zeile, "G1 Z%0.3Lf\n", Zrun);
                              fwrite(Zeile, 1, strlen(Zeile), FileOut);
                              firstGearPath = 0;
                            }
                            sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x - minX), (y - minY));
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                            if(closePathX == -99999.9 && closePathY == -99999.9){
                              closePathX = x;
                              closePathY = y;
                            }
                          }
                          x = r * cosl(PI + PI / gearTeeth + beta) + lastX - firstX;
                          y = r * sinl(PI + PI / gearTeeth + beta) + lastY - firstY;
                          lastX = x;
                          lastY = y;
                          sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x - minX), (y - minY));
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          firstX = 0.0;
                          firstY = 0.0;
                          r = (toothDiameter - gearMiller);
                          for(alpha = 2.0 * PI - PI / gearTeeth / 2.0 + PI / gearTeeth + beta; alpha > PI + PI / gearTeeth / 2.0 + PI / gearTeeth + beta; alpha-=0.1){
                            if(firstX == 0.0 && firstY == 0.0){
                              firstX = r * cosl(alpha);
                              firstY = r * sinl(alpha);
                            }
                            x = r * cosl(alpha) + lastX - firstX;
                            y = r * sinl(alpha) + lastY - firstY;
                            sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x - minX), (y - minY));
                            fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          }
                          x = r * cosl(PI + PI / gearTeeth / 2.0 + PI / gearTeeth + beta) + lastX - firstX;
                          y = r * sinl(PI + PI / gearTeeth / 2.0 + PI / gearTeeth + beta) + lastY - firstY;
                          lastX = x;
                          lastY = y;
                          averageX+=x;
                          averageY+=y;
                          coordNumber+=1.0;
                          sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x - minX), (y - minY));
                          fwrite(Zeile, 1, strlen(Zeile), FileOut);
                          firstX = -99999.0;
                          firstY = -99999.0;
                        }//for(i = 0.0; i < gearTeeth; i+=1.0){

                        x=closePathX;
                        y=closePathY;
                        sprintf(Zeile, "G1 X%0.3Lf Y%0.3Lf\n", (x - minX), (y - minY));
                        fwrite(Zeile, 1, strlen(Zeile), FileOut);

                      }//while(cabs(Zrun) < materialThickness){

                      sprintf(Zeile, "G1 Z6.000\n");
                      fwrite(Zeile, 1, strlen(Zeile), FileOut);
                    }
                    sprintf(Zeile, "G0 X0.000 Y0.000\n");
                    fwrite(Zeile, 1, strlen(Zeile), FileOut);
                    fclose(FileOut);

                    while(kbhit()){
                      getch();
                    }
                    MessageText("File 'GearWheel.svg successfully created. Press any key to continue...", 1, MessageY, 0);
                    getch();

                  }//if(deeperPerRun > 0.0){
                }//if(materialThickness > 0.0){
              }//if(gearMiller > 0.0){
            }//if(gearHole > 0.0){
          }//if(gearTeeth > 0.0){
        }//if(gearModule > 0.0){
      }//if(KeyHit == 'g'){





      if(KeyHit == 'p'){//Plot file
        //MessageText("13 seconds until plotting starts !!!!!!!!!!!!!!!!!", 1, 20, 0);
        //sleep(10);
        //MessageText("3 seconds until plotting starts !!!!!!!!!!!!!!!!!", 1, 20, 0);
        //sleep(3);
        if(kbhit()){
          getch();
          KeyHit = 27;
        }
        if(strcmp(FileName, "noFiLE") != 0){
          if((GcodeFile=fopen(FullFileName,"rb"))==NULL){
            sprintf(TextLine, "Can't open file '%s'!\n", FullFileName);
            strcpy(FileName, "NoFiLE");
            ErrorText(TextLine);
          }
        }
        if(strcmp(FileName, "noFiLE") != 0 && KeyHit != 27){
          currentX = 0;
          currentY = 0;
          currentZ = 0;
          newX = 0;
          newY = 0;
          newZ = 0;
          PlotStartTime = time(0);
          coordinateCount = 0;
          stopPlot = 0;
          latestFeedRate = MAX_FEED_RATE;
          currentFeedRate = MAX_FEED_RATE;

          while (GetGcodeCoordinates(0, 1, 0) > 0 && stopPlot == 0){

            if(currentFeedRate != feedRateNew){
              StepPause = 1000.0 / (StepsPermmX * feedRateNew  / 60.0);

              while (StepPauseArduino < StepPause){
                sendport('S');//Raise step pause
                StepPauseArduino+=1;
              }

              while(StepPauseArduino > StepPause){
                sendport('s');//Reduce step pause
                StepPauseArduino-=1;
              }

              currentFeedRate = feedRateNew;
            }

            if(newX != currentX || newY != currentY || newZ != currentZ){
              if(CalculatePlotter(newX - currentX, newY - currentY, newZ - currentZ) == 1){
                stopPlot = 1;
              }
              coordinateCount++;
              PrintMenue_03(FullFileName, coordinateCount, maxCoordinateCount, newX, newY, newZ, PlotStartTime);
            }

          }

          fclose(GcodeFile);
          stopPlot = 0;
          PrintMenue_03(FullFileName, coordinateCount, maxCoordinateCount, newX, newY, newZ, PlotStartTime);
          CalculatePlotter( 0, 0, -currentZ + 10 * StepsPermmZ);
          CalculatePlotter( -currentX, -currentY, 0);
          currentX = 0;
          currentY = 0;
          while(kbhit()){
            getch();
          }
          //Set machine to maximum speed
          currentFeedRate = MAX_FEED_RATE;
          StepPause = 1000.0 / (StepsPermmX * feedRateNew  / 60.0);
          while (StepPauseArduino < StepPause){
            sendport('S');//Raise step pause
            StepPauseArduino+=1;
          }
          while(StepPauseArduino > StepPause){
            sendport('s');//Reduce step pause
            StepPauseArduino-=1;
          }

          MessageText("Finished! Press any key to return to main menu.", MessageX, MessageY, 0);
          getch();
          PrintMenue_01();

        }//if(strcmp(FileName, "noFiLE") != 0){
      }//if(KeyHit == 'p'){


    }//if(MenueLevel == 0){

    if(MenueLevel == 1){//Select file
      KeyControl(0);
      if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 66 && KeyCode[3] == 0 && KeyCode[4] == 0){
        if(FileSelected < FilesFound - 1){
          FileSelected++;
          if(FileSelected > MaxFileRows - 2){
            FileStartRow = FileSelected - MaxFileRows + 2;
          }
          strcpy(FileName, PrintMenue_02(FileStartRow, FileSelected));
        }
      }

      if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 65 && KeyCode[3] == 0 && KeyCode[4] == 0){
        if(FileSelected > 0){
          if(FileSelected == FileStartRow + 1){
            if(FileStartRow > 0){
              FileStartRow--;
            }
          }
          FileSelected--;
          strcpy(FileName, PrintMenue_02(FileStartRow, FileSelected));
        }
      }

      if(KeyHit == 10){//Read file and store values
        MenueLevel = 0;
        clrscr(MessageY + 1, MessageY + 1);
        strcpy(FullFileName, PicturePath);
        strcat(FullFileName, "/");
        strcat(FullFileName, FileName);
        if((GcodeFile=fopen(FullFileName,"rb"))==NULL){
          sprintf(TextLine, "Can't open file '%s'!\n", FullFileName);
          ErrorText(TextLine);
          strcpy(FileName, "NoFiLE");
        }
        else{
          printf("Opened file '%s'!\n", FullFileName);

          xMin=1000000;
          xMax=-1000000;
          yMin=1000000;
          yMax=-1000000;
          zMin=1000000;
          zMax=-1000000;

          newX = -99999999;
          newY = -99999999;
          newZ = -99999999;
          maxCoordinateCount = 0;

          while (GetGcodeCoordinates(0, 0, 0) > 0){

            if(newX > xMax && newX != -99999999){
              xMax = newX;
            }
            if(newX < xMin && newX != -99999999){
              xMin = newX;
            }

            if(newY > yMax && newY != -99999999){
              yMax = newY;
            }
            if(newY < yMin && newY != -99999999){
              yMin = newY;
            }

            if(newZ > zMax && newZ != -99999999){
              zMax = newZ;
            }
            if(newZ < zMin && newZ != -99999999){
              zMin = newZ;
            }

            maxCoordinateCount++;
          }

          fclose(GcodeFile);
          Scale = 1.0;
          printf("xMax = %ld, xMin=%ld, yMax = %ld, yMin = %ld  ", xMax, xMin, yMax, yMin);
          getch();
        }
        PrintMenue_01();
      }//if(KeyHit == 10){

    }//if(MenueLevel == 1){


    if(KeyHit == 27){
      if(MenueLevel == 0){
        clrscr(MessageY + 1, MessageY + 1);
        MessageText("Exit program (y/n)?", MessageX, MessageY + 1, 0);
        while(KeyHit != 'y' && KeyHit != 'n'){
          KeyHit = getch();
          if(KeyHit == 'y'){
            exit(0);
          }
        }
      }
      if(MenueLevel == 1){
        MenueLevel = 0;
        strcpy(FileName, FileNameOld);
        PrintMenue_01();
      }
      clrscr(MessageY + 1, MessageY + 1);
    }
  }

  return 0;
}
